"""
yamlconfig.py
A config file reader that support yaml based config files.

Copyright 2010, Kelsey Hightower
Kelsey Hightower <kelsey.hightower@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301  USA
"""
import yaml

from string import Template


class YamlConfig(dict):
    """
    YAML configuration file reader with basic variable interpolation.
    """

    def __init__(self, infile, defaults=None):
        """
        Initialization method.
        """
        dict.__init__(self)
        self.infile = infile

        if defaults is None:
            self.defaults = {}
        else:
            self.defaults = defaults

        indict = self.get_config()

        for entry in indict:
            self[entry] = indict[entry]

    def _interpolate(self):
        """
        Interpolate variables.
        """
        with open(self.infile, 'r') as config:
            template = Template(config.read())
            interpolated_config = template.safe_substitute(self.defaults)
        return interpolated_config

    def _deserialize(self):
        """
        Deserialize YAML string.
        """
        config = yaml.load(self.infile)
        return config

    def get_config(self):
        """
        Parse and return the configuration object.
        """
        self.infile = self._interpolate()
        config = self._deserialize()
        return config



import os
import unittest2
import yamlconfig

dummy_yaml = """
main:
    name: 'Napoleon'
    age: 15
    skills: ['Ninja', 'Dancing', 'Nunchucks', $default_skills]
    net_worth: $10
"""

def create_dummy_config():
    with open('dummy_config.yaml', 'w') as config:
        config.write(dummy_yaml)

class TestYamlConfig(unittest2.TestCase):

    def setUp(self):
        create_dummy_config()
        self.infile = 'dummy_config.yaml'
        self.defaults = {'default_skills': 'Kickball'} 
        self.config = yamlconfig.YamlConfig(self.infile, self.defaults)

    def tearDown(self):
        os.remove('dummy_config.yaml')

    def test_get_config(self):
        expected_config = {
            'main': {
                'name': 'Napoleon',
                'age': 15,
                'skills': ['Ninja', 'Dancing', 'Nunchucks', 'Kickball'],
                'net_worth': '$10'
                }
        }
        self.assertDictEqual(expected_config, self.config)

    def test_list_type(self):
        self.assertIsInstance(self.config['main']['skills'], list)

    def test_int_type(self):
        self.assertIsInstance(self.config['main']['age'], int)

    def test_str_type(self):
        self.assertIsInstance(self.config['main']['name'], str)

    def test_keyerror(self):
        def getoption(option):
            return self.config['main'][option]
        self.assertRaises(KeyError, getoption, 'girlfriend')

    def test_safe_substitute(self):
        self.assertEqual(self.config['main']['net_worth'], '$10')

    def test_with_no_defaults(self):
        create_dummy_config()
        infile = 'dummy_config.yaml'
        config = yamlconfig.YamlConfig(infile)
        expected_config = {
            'main': {
                'name': 'Napoleon',
                'age': 15,
                'skills': ['Ninja', 'Dancing', 'Nunchucks', '$default_skills'],
                'net_worth': '$10'
                }
        }
        self.assertDictEqual(expected_config, config)

if __name__ == '__main__':
    unittest2.main()



from distutils.core import setup

setup(
    name='yamlconfig',
    version='0.3.1',
    description='YAML configuration file reader with basic variable interpolation.',
    author='Kelsey Hightower',
    author_email='kelsey.hightower@gmail.com',
    url='https://bitbucket.org/khightower/yamlconfig',
    py_modules=['yamlconfig'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
    install_requires=['PyYAML >= 3.0']
    )
